
/*
 *  sample.c
 *  Copyright (c) 1999 by Brad Proctor.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 */

/*
 * $Id: sample.c,v 1.5 1999/06/21 15:04:55 brad Exp $
 */

/* This utility breaks the ANSI/ISO C only rule that I have been following
 * by using the getline() function.  I will change this later to use either
 * getline() or fgets().
 */

#define _GNU_SOURCE     /* We need getline() */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "version.h"

#define invalid() \
{                 \
   printf("%s: Invalid option: `%s'\n", argv[0], argv[i]);       \
   fputs("Try `sample --help' for more information.\n", stdout); \
   return (1);   \
}

#define argument(a) \
{                   \
   printf("%s: Option `"a"' requires an argument.\n", argv[0]); \
   fputs("Try `sample --help' for more infomation.\n", stdout); \
   return (1);     \
}

static char help_text[] =
{
   "Usage: sample [options]\n\n"
   "  -l n       Prints every nth line\n"
   "  -c n       Prints every nth character\n"
   "  --help     Print this help message\n"
   "  --version  Print the program version\n\n"
   "Report bugs to fredlie@sprynet.com"
};

/*
 * Output on a per line basis
 */

static inline void sample_lines(int lines)
{
   int i = lines;
   int bytes = 0;
   char *ptr = NULL;
   while(1)
   {  
      if(getline(&ptr, &bytes, stdin) == -1)
         return;
      if(--i <= 0)
      {
         fputs(ptr, stdout);
         i = lines;
      }
   }
}

/*
 * Output on a per character basis
 */

static inline void sample_chars(int chars)
{
   int i = chars;
   int c = getchar();
   
   while(c != EOF)
   {
      if(--i <= 0)
      {
         putchar(c);
         i = chars;
      } 
      c = getchar();
   }
}

/*
 * Main
 */

int main(int argc, char *argv[])
{
   int i, sample;
   char *ptr;

   for(i = 1; i < argc; ++i)
   {
      ptr = argv[i];
      if(*ptr == '-')
      {
         switch(*++ptr)
         {
            case '-':
               ++ptr;
               if(!strcmp(ptr, "help"))
               {
                  puts(help_text);
                  return (0);
               }
               else if(!strcmp(ptr, "version"))
               {
                  puts("sample (butils) "VERSION_STRING);
                  return (0);
               }
               else
                  invalid();
            case 'l':
               if(++i < argc)
                  sscanf(argv[i], "%i", &sample);
               else
                  argument("-l");
               sample_lines(sample);
               return (0);
            case 'c':
               if(++i < argc)
                  sscanf(argv[i], "%i", &sample);
               else
                  argument("-c");
               sample_chars(sample); 
               return (0);   
            default:
               invalid();
         }
      }
      else
         invalid();
   }
   return (0);
}
